from django.utils import simplejson
from dajaxice.decorators import dajaxice_register
import os
from django.contrib.gis.utils import LayerMapping
from models import Area, Place, City
import random
from django.contrib.gis.geos import Point, MultiPolygon, Polygon

#method that save a shapefile
@dajaxice_register
def saveShapefile(request,data):
    #column that the user have selected
    column = data[0]
    #path of the .shp
    path = data[1]
    #name of the city
    cityName = data[2]
    try:
        #if the city already exists we delete the areas and the city
        Area.objects.filter(city__name=cityName).delete()
        City.objects.get(name=cityName).delete()
    finally:
        #save the shapefile data in the database
        mapping = {'name' : column, 'poly' : 'POLYGON'}
        lm = LayerMapping(Area, path, mapping)
        lm.save(verbose=True)
        polyList = []
        areaPolyList = []
        areaNameList = []
        #the new areas aren't related with any city
        for area in Area.objects.filter(city__isnull=True):
            #we save all the polygons in a list
            polyList.append(area.poly)
            areaNameList.append(area.name)
            areaPolyList.append(str(area.poly))
        #we make a multipolygon with all the polygons
        mp = MultiPolygon(polyList)
        #we create a new city with the name and the multipolygon
        newCity = City(name=cityName,poly=mp.convex_hull)
        newCity.save()
        #we update the areas with the new city
        Area.objects.filter(city__isnull=True).update(city=newCity)
        cityPolyList = []
        cityPolyList.append(str(City.objects.get(name=cityName).poly));
        cityNameList = []
        cityNameList.append(cityName)
        return simplejson.dumps({'polyArea': areaPolyList,
                                 'nameArea': areaNameList,
                                 'polyCity': cityPolyList,
                                 'nameCity': cityNameList,})
        
                      
    
#method that save a shapefile considering previous polygons
@dajaxice_register
def saveExistingShapefile(request, data):
    try:
        column = data[0]
        path = data[1]
        cityName = data[2]
        mapping = {'name' : column, 'poly' : 'POLYGON'}
        lm = LayerMapping(Area, path, mapping)
        lm.save(verbose=True)
        polyList = []
        for area in Area.objects.filter(city__isnull=True):
            polyList.append(area.poly)
        for area in Area.objects.filter(city__name=cityName):
            #we save the areas that already exist and we attach to the new areas
            polyList.append(area.poly)
        mp = MultiPolygon(polyList)
        #we update the city polygon
        City.objects.filter(name=cityName).update(poly=mp.convex_hull)
        #we link the new areas with the city
        Area.objects.filter(city__isnull=True).update(city=City.objects.get(name=cityName))
        return simplejson.dumps({'message' : 'The shapefiles has been added'})
    except:
        return simplejson.dumps({'message' : 'Something went wrong, try with another column'})

    

    
#save the images giving the coordinates and if it's a world point 
@dajaxice_register
def generateImage(request, data):
        if data[2]=="world":
            pnt = Point(float(data[1]),float(data[0]))
            urlGoogle =  "http://maps.googleapis.com/maps/api/streetview?size=400x400&location="+str(pnt.y)+",%20"+str(pnt.x)+"&sensor=false&key=AIzaSyBTdISjiOW54zc95uWK4sQYtZWOJlivio0"
            p = Place(url = urlGoogle, point = pnt, isFakePoint = True)
            #do not save it if it is a 'noimage'
            check = p.cache()
            if (check):
                return simplejson.dumps({'message': 'ok'})
            else:
                return simplejson.dumps({'message': 'not ok'})
        else:    
            pnt = Point(float(data[1]),float(data[0]))
            urlGoogle =  "http://maps.googleapis.com/maps/api/streetview?size=400x400&location="+str(pnt.y)+",%20"+str(pnt.x)+"&sensor=false&key=AIzaSyBTdISjiOW54zc95uWK4sQYtZWOJlivio0"
            p = Place(url = urlGoogle, point = pnt, city = City.objects.get(name = data[2]), isFakePoint = True)
            check = p.cache()
            if (check):
                return simplejson.dumps({'message': 'ok'})
            else:
                return simplejson.dumps({'message': 'not ok'})


#generate Google Street View images for the world
@dajaxice_register
def generateImageWorld(request, data, country):
        aux = 0
        #number images
        while (aux < int(data[0])):
            countryTmp = country[int(random.random()*len(country))];
            countryTmp['lng'] = countryTmp['lng'] + float(((random.random()-random.random())/100)*2)
            countryTmp['lat'] = countryTmp['lat'] + float(((random.random()-random.random())/100))
            pnt = Point(countryTmp['lng'],countryTmp['lat'])
            urlGoogle =  "http://maps.googleapis.com/maps/api/streetview?size=400x400&location="+str(countryTmp['lat'])+",%20"+str(countryTmp['lng'])+"&sensor=false&key=AIzaSyBTdISjiOW54zc95uWK4sQYtZWOJlivio0"
            p = Place(url = urlGoogle, point = pnt)
            check = p.cache()
            if (check):
                aux = aux + 1
        return aux
    
    
#function that generate random points inside a given areas or city
@dajaxice_register
def generateRandomPoints(request, data):
    numpoints = int(data[0])
    cityName = data[1]
    typeMap = data[2]
    drop = data[3]
    strPoints = ""
    if drop == 'True':
        Place.objects.filter(city__name=cityName, isFakePoint=False).delete()
   
    #if its a city we make points for the whole city
    if typeMap == "City":
        i =  City.objects.get(name=cityName)
        suma = 0
        #get random point
        extent = i.poly.extent
        pnt = get_random_point(extent)
        while (suma < numpoints):
            #check if the point is inside the polygon
            if (i.poly.contains(pnt)):
                urlGoogle =  "http://maps.googleapis.com/maps/api/streetview?size=400x400&location="+str(pnt.y)+",%20"+str(pnt.x)+"&sensor=false&key=AIzaSyBTdISjiOW54zc95uWK4sQYtZWOJlivio0"
                p = Place(url = urlGoogle, point = pnt, city = City.objects.get(name = cityName),isFakePoint=False)
                check = p.cache()
                #check if the image exists
                if (check):
                    #add the point to a string that will be returned
                    strPoints = strPoints + "AA" + str(pnt.y) +","+ str(pnt.x);
                    suma = suma + 1
                    
            pnt = get_random_point(extent)
        return simplejson.dumps({'message': strPoints})
    #if it's area we generate n points for each area
    if typeMap == "Area":
        #n points for each area
        for i in Area.objects.filter(city__name=cityName):
            suma = 0
            extent = i.poly.extent
            pnt = get_random_point(extent)
            while (suma < numpoints):
                if (i.poly.contains(pnt)):
                    urlGoogle =  "http://maps.googleapis.com/maps/api/streetview?size=400x400&location="+str(pnt.y)+",%20"+str(pnt.x)+"&sensor=false&key=AIzaSyBTdISjiOW54zc95uWK4sQYtZWOJlivio0"
                    p = Place(url = urlGoogle, point = pnt, city = City.objects.get(name = cityName), area = i,isFakePoint=False)
                    check = p.cache()
                    if (check):
                        strPoints = strPoints + "AA" + str(pnt.y) +","+ str(pnt.x);
                        suma = suma + 1
                    
                pnt = get_random_point(extent)
    
        return simplejson.dumps({'message': strPoints})
    
        
        
    
#generate a random point giving the extension of a box
def get_random_point(extent):
    xrange_ = extent[2] - extent[0]
    yrange_ = extent[3] - extent[1]
    randx = xrange_ * random.random() + extent[0]
    randy = yrange_ * random.random() + extent[1]
    return Point(randx, randy)
    
    

    
