from django.db import models
from django.contrib.gis.db import models
import urllib
from django.core.files import File
import zipfile, os, os.path, shutil
from OUOadmin import settings
from django.contrib import messages
from django.contrib.gis.utils import LayerMapping

#path OS-independent
SITE_ROOT = os.path.dirname(os.path.realpath(__file__))
DJANGO_ROOT = os.path.abspath(os.path.join(SITE_ROOT, '..'))

#Define a whole city
class City(models.Model):
    name = models.CharField(max_length=50, unique=True)
    poly = models.PolygonField()
    # Returns the string representation of the model.
    def __unicode__(self):
        return self.name
    
    class Meta:
        verbose_name_plural = "Cities"

#Define diferent areas of a city
class Area(models.Model):
    name = models.CharField(max_length=50)
    poly = models.PolygonField()
    city = models.ForeignKey(City, null=True)  
    # Returns the string representation of the model.
    def __unicode__(self):
        return self.name

#Class used to upload a shapefile, it doesn't have direct acces on the interface
class ShapeFileZip(models.Model):
    zip = models.FileField(upload_to=os.path.join(DJANGO_ROOT, 'shapes'))
    name = models.CharField(max_length=255, null=True, blank=True)
    
    #override the save method
    def save(self, *args, **kwargs):
        try:
            ShapeFileZip.objects.all().delete()
        finally:
            #check if it's a zip file
            if ".zip" in self.zip.name:
                super(ShapeFileZip, self).save(*args, **kwargs)
    
                # Convert file and dir into absolute paths
                fullpath = os.path.join(os.path.join(DJANGO_ROOT, 'shapes'),self.zip.name)
                dirname = os.path.dirname(fullpath)
                
                # Get a real Python file handle on the uploaded file
                fullpathhandle = open(fullpath, 'r') 
    
                # Unzip the file, creating subdirectories as needed
                zfobj = zipfile.ZipFile(fullpathhandle)
                for name in zfobj.namelist():
                    if name.endswith('.shp'):
                        self.name = os.path.join(os.path.join(DJANGO_ROOT, 'shapes'),name)
                        super(ShapeFileZip, self).save(*args, **kwargs)
                                                
                    if name.endswith('/'):
                        try: # Don't try to create a directory if exists
                            os.mkdir(os.path.join(dirname, name))
                        except:
                            pass
                    else:
                        outfile = open(os.path.join(dirname, name), 'wb')
                        outfile.write(zfobj.read(name))
                        outfile.close()
                    
                # Now try and delete the uploaded .zip file and the 
                # stub __MACOSX dir if they exist.
                try:
                    os.remove(fullpath)
                except:
                    pass
                    
                try:
                    osxjunk = os.path.join(dirname,'__MACOSX')
                    shutil.rmtree(osxjunk)
                except:
                    pass
    
#Define a geographical point in the world
class Place(models.Model):
    point = models.PointField();
    url = models.CharField(max_length=255, null=True, blank=True)
    photo = models.ImageField(upload_to=os.path.join(DJANGO_ROOT, 'images'), blank=True, null=True)
    city = models.ForeignKey(City, null=True, blank=True)
    area = models.ForeignKey(Area, null=True, blank=True)
    isFakePoint = models.NullBooleanField()
            
    #update the city 
    def resetCity(self):
        for city in City.objects.all():
            print city.poly.contains(self.point)
            if(city.poly.contains(self.point)):
                self.city = city
                self.save()
           
    #update the area 
    def resetArea(self):
        for area in Area.objects.all():
            if(area.poly.contains(self.point)):
                self.area = area
                self.save()
    
    
    #check if an image can be saved and delete if it can't
    def cache(self):
        #retrieve the image
        result = urllib.urlretrieve(self.url)
        #name and save the image
        #we save the image name as the coordinates
        self.photo.save(
                    os.path.basename(self.url)[32:64],
                    File(open(result[0]))
                    )
        #save the object because we need it to compare
        self.save()
        #if it a 'noimage' we delete it
        if(self.is_equal()):
            self.deletePhoto()
            return False
        return True
        
    def is_equal(self):
        #'noimage'
        f1 = open(os.path.join(DJANGO_ROOT, 'noimage'), 'r')
        #image already saved
        tmp = "%s" % (self.photo)
        f2 = open(tmp , 'r')
        
        #comparsion
        if f1.read() == f2.read():
            return True
        else:
            return False
    
    def deletePhoto(self, *args, **kwargs):
        # You have to prepare what you need before delete the model
        storage, path = self.photo.storage, self.photo.path
        # Delete the model before the file
        super(Place, self).delete(*args, **kwargs)
        # Delete the file after the model
        storage.delete(path)
