from django.http import HttpResponse, HttpResponseNotFound, HttpResponseRedirect
from django.template import Context, loader
from models import Area, City, ShapeFileZip, Place
from django.contrib.admin.views.decorators import staff_member_required
from django.shortcuts import render_to_response
from django.template import RequestContext
from forms import zipForm
from django.contrib.gis.gdal import DataSource

#Generate easy-to-guess it's a veiw that displays well-known points of a city
@staff_member_required  
def generateetg (request):
    if request.method == 'POST':
        #depending on the page that called the view we have diferent post
        #we delete the previous Places
        drop = False
        try:
            cityName = request.POST['city']
        except:
            cityName = request.POST.getlist('cityOptions')[0]
            
        if request.POST.get('drop', False):
            print "I'm here"
            drop = True
            Place.objects.filter(city__name=cityName).delete()

                    
        cityPolyList = []
        #polygon of the whole city
        cityPolyList.append(str(City.objects.get(name=cityName).poly));
        return render_to_response('generateetg.html', {
                'city' : cityName,
                'cityPoly' : cityPolyList,
                'drop' : drop
            }, context_instance=RequestContext(request) )
        
        
#generate random points for the city
@staff_member_required        
def generateRandom (request):
    if request.method == 'POST':
        cityName = request.POST['city']
        #type is already used in Python
        typeTmp = request.POST['type']
        npoints = request.POST['npoints']
        drop = request.POST['drop']
        polyList = []
        nameList = []
        #depending on the type that the administrator selected we return diferent polygons
        if typeTmp == "Area":
            for area in Area.objects.filter(city__name=cityName):
                polyList.append(area.poly)
                nameList.append(area.name)
        else:
            nameList.append(cityName);
            polyList.append(str(City.objects.get(name=cityName).poly));
        return render_to_response('generateRandom.html', {
                'city' : cityName,
                'type' : typeTmp,
                'polyList' : polyList,
                'nameList' : nameList,
                'npoints' : npoints,
                'drop' : drop
            }, context_instance=RequestContext(request) )
    
    
    
#The view for the import of a shapefile for a new city
@staff_member_required  
def importShapefileNewCity(request):
    #if the request is a GET we only display the zip form and the city that the user selected
    if 'city' in request.GET:
        form = zipForm()
        return render_to_response('importShapefileNewCity.html', { 'form' : form,
                'city' : request.GET['city']
        }, context_instance=RequestContext(request) )
    #if the request is a POST we have to manage the zip file
    elif request.method == 'POST':
        cityName = request.POST['cityH']
        form = zipForm(request.POST, request.FILES)
        #check if it's valid
        if form.is_valid():
            #check if it's a zip file
            zipFile = ShapeFileZip(zip = request.FILES['zip'])
            zipFile.save()
            #we open the shapefile
            ds = DataSource(zipFile.name)
            layer = ds[0]
            listFields = []
            #that checkes if the geotype of the shapefile is a polygon
            if (layer.geom_type == "POLYGON"):
                #the list of possible fields that the shapefile have
                listFields = layer.fields
                return render_to_response('importShapefileNewCity.html', { 'form' : form,
                                                                          'listFields' : listFields,
                                                                          'city': cityName,
                                                                          'zipFile':zipFile,
                    }, context_instance=RequestContext(request) )
            else:
                return HttpResponseNotFound('<h1>The geometry type is not a polygon</h1>') 


# Like the previous view with a different HTML
@staff_member_required     
def importShapefile(request):
    if request.method == 'GET':
        form = zipForm()
        cities = list(City.objects.all()[:])
        return render_to_response('importShapefile.html', {
                                                           'form' : form,
                                                           'cities' : cities
                }, context_instance=RequestContext(request) )
    elif request.method == 'POST':
        cityName = request.POST['cityOptions']
        form = zipForm(request.POST, request.FILES)
        if form.is_valid():
            zipFile = ShapeFileZip(zip = request.FILES['zip'])
            zipFile.save()
            ds = DataSource(zipFile.name)
            layer = ds[0]
            listFields = []
            if (layer.geom_type == "POLYGON"):
                listFields = layer.fields
                return render_to_response('importShapefile.html', { 'form' : form,
                                                                          'listFields' : listFields,
                                                                          'city': cityName,
                                                                          'zipFile':zipFile,
                    }, context_instance=RequestContext(request) )
            else:
                return HttpResponseNotFound('<h1>The geometry type is not a polygon</h1>') 


@staff_member_required
def createCity(request):
    cities = list(City.objects.all()[:])
    return render_to_response('createCity.html', {
                                                  'cities' : cities
                                                  }, context_instance=RequestContext(request) )

@staff_member_required     
def generatePoints(request):
    cities = list(City.objects.all()[:])
    return render_to_response('generatePoints.html', {
                'cities' : cities
            }, context_instance=RequestContext(request) )
    
@staff_member_required     
def worldPoints(request):
    return render_to_response('worldPoints.html', {
            }, context_instance=RequestContext(request) )
    