from django.template import RequestContext
from django.shortcuts import render_to_response
from adminShapes.models import City, Place
from quiz.models import Answer
from django.contrib.gis.geos import Point
from django.contrib import messages
from django.http import HttpResponseRedirect
from django.core.urlresolvers import reverse

def start_game(request):
  city = City.objects.order_by('?')[0]
  city_name = city.name.replace(" ", "-")
  city_name = city_name.lower()

  return HttpResponseRedirect(reverse('quiz.views.play', kwargs={'city_name': city_name}))

def play(request, city_name):
  city_name = city_name.replace("-", " ")
  city = City.objects.get(name__iexact=city_name)

  # City bounding box to bias the user search to the city
  bb = city.poly.extent

  # City center
  lat = str(city.poly.point_on_surface.y)
  lon = str(city.poly.point_on_surface.x)

  return render_to_response('quiz/play.html', 
                            {'city': {'name': city.name,
                                      'bb': bb, 
                                      'id': city.id,
                                      'center': {'lat':lat, 'lon':lon}}},
                            context_instance=RequestContext(request))

def send_answer(request):
  # Received data
  place_id = request.POST['place_id']
  lat = float(request.POST['answer_lat'])
  lon = float(request.POST['answer_lon'])

  # Add answer to table
  point = Point(lon, lat)
  place = Place.objects.get(pk=place_id)
  answer = Answer(place=place, point=point)
  answer.save()

  # Get city being played to return to the right URL
  city_name = place.city.name.replace(" ", "-")
  city_name = city_name.lower()

  messages.success(request, "Your distance was {0}".format(answer.city_distance()))
  return HttpResponseRedirect(reverse('quiz.views.play', kwargs={'city_name': city_name}))
                            
